@@ -227,7 +227,7 @@ EXPORTED int conversations_open_path(const char *fname, struct conversations_sta
                    &val, &vallen, &open->s.txn)) {
         struct dlist *dl = NULL;
         struct dlist *dp;
-        dlist_parsemap(&dl, 0, val, vallen);
+        dlist_parsemap(&dl, 0, 0, val, vallen);
         for (dp = dl->head; dp; dp = dp->next) {
             strarray_append(open->s.folder_names, dlist_cstring(dp));
         }
@@ -953,7 +953,7 @@ EXPORTED int conversation_parsestatus(const char *data, size_t datalen,
         return IMAP_MAILBOX_BADFORMAT;
     }
 
-    r = dlist_parsemap(&dl, 0, rest, restlen);
+    r = dlist_parsemap(&dl, 0, 0, rest, restlen);
     if (r) return r;
 
     n = dl->head;
@@ -1104,7 +1104,7 @@ EXPORTED int conversation_parse(struct conversations_state *state,
 
     if (version != CONVERSATIONS_VERSION) return IMAP_MAILBOX_BADFORMAT;
 
-    r = dlist_parsemap(&dl, 0, rest, restlen);
+    r = dlist_parsemap(&dl, 0, 0, rest, restlen);
     if (r) return r;
 
     conv = conversation_new(state);
@@ -1842,7 +1842,6 @@ EXPORTED int conversations_update_record(struct conversations_state *cstate,
     int *delta_counts = NULL;
     int i;
     modseq_t modseq = 0;
-    const struct index_record *record = NULL;
     int r = 0;
 
     if (old && new) {
@@ -1864,21 +1863,15 @@ EXPORTED int conversations_update_record(struct conversations_state *cstate,
         }
     }
 
+    const struct index_record *record = new ? new : old;
     if (new && !old) {
         /* add the conversation */
         r = mailbox_cacherecord(mailbox, new); /* make sure it's loaded */
         if (r) return r;
         r = message_update_conversations(cstate, new, &conv);
         if (r) return r;
-        record = new;
-        /* possible if silent (i.e. replica) */
-        if (!record->cid) return 0;
     }
-    else {
-        record = new ? new : old;
-        /* skip out on non-CIDed records */
-        if (!record->cid) return 0;
-
+    else if (record->cid) {
         r = conversation_load(cstate, record->cid, &conv);
         if (r) return r;
         if (!conv) {
@@ -1893,6 +1886,24 @@ EXPORTED int conversations_update_record(struct conversations_state *cstate,
         }
     }
 
+    // always update the GUID information first, as it's used for search
+    // even if conversations have not been set on this email
+    if (new) {
+        if (!old) {
+            r = conversations_set_guid(cstate, mailbox, new, /*add*/1);
+            if (r) return r;
+        }
+    }
+    else {
+        if (old) {
+            r = conversations_set_guid(cstate, mailbox, old, /*add*/0);
+            if (r) return r;
+        }
+    }
+
+    // the rest is bookkeeping for CIDs only
+    if (!record->cid) return 0;
+
     if (cstate->counted_flags)
         delta_counts = xzmalloc(sizeof(int) * cstate->counted_flags->count);
 
@@ -1928,10 +1939,6 @@ EXPORTED int conversations_update_record(struct conversations_state *cstate,
         }
         delta_num_records--;
         modseq = MAX(modseq, old->modseq);
-        if (!new) {
-            r = conversations_set_guid(cstate, mailbox, old, /*add*/0);
-            if (r) return r;
-        }
     }
 
     if (new) {
@@ -1953,10 +1960,6 @@ EXPORTED int conversations_update_record(struct conversations_state *cstate,
         }
         delta_num_records++;
         modseq = MAX(modseq, new->modseq);
-        if (!old) {
-            r = conversations_set_guid(cstate, mailbox, new, /*add*/1);
-            if (r) return r;
-        }
     }
 
     /* XXX - combine this with the earlier cache parsing */
