@@ -414,7 +414,7 @@ static int sieve_redirect(void *ac,
     /* if we have a msgid, we can track our redirects */
     if (m->id) {
         snprintf(buf, sizeof(buf), "%s-%s", m->id, rc->addr);
-        sievedb = make_sieve_db(mbname_userid(sd->mbname));
+        sievedb = make_sieve_db(mbname_recipient(sd->mbname, ((deliver_data_t *) mc)->ns));
 
         dkey.id = buf;
         dkey.to = sievedb;
@@ -496,7 +496,7 @@ static int sieve_reject(void *ac,
     body = msg_getheader(md, "original-recipient");
     origreceip = body ? body[0] : NULL;
     if ((res = send_rejection(md->id, md->return_path,
-                              origreceip, mbname_userid(sd->mbname),
+                              origreceip, mbname_recipient(sd->mbname, ((deliver_data_t *) mc)->ns),
                               rc->msg, md->data)) == 0) {
         snmp_increment(SIEVE_REJECT, 1);
         syslog(LOG_INFO, "sieve rejected: %s to: %s",
@@ -735,7 +735,7 @@ static int send_response(void *ac,
     while (waitpid(sm_pid, &sm_stat, 0) < 0);
 
     if (sm_stat == 0) { /* sendmail exit value */
-        sievedb = make_sieve_db(mbname_userid(sdata->mbname));
+        sievedb = make_sieve_db(mbname_recipient(sdata->mbname, ((deliver_data_t *) mc)->ns));
 
         dkey.id = outmsgid;
         dkey.to = sievedb;
@@ -832,13 +832,6 @@ sieve_interp_t *setup_sieve(void)
     return interp;
 }
 
-static void _rm_dots(char *p)
-{
-    for (; *p; p++) {
-        if (*p == '.') *p = '^';
-    }
-}
-
 static int sieve_find_script(const char *user, const char *domain,
                              const char *script, char *fname, size_t size)
 {
@@ -875,10 +868,7 @@ static int sieve_find_script(const char *user, const char *domain,
         }
         else {
             char hash = (char) dir_hash_c(user, config_fulldirhash);
-            char *usercopy = xstrdup(user);  // stupid hashing of names, we SHOULD just allow dots on disk
-            _rm_dots(usercopy);
-            len += snprintf(fname+len, size-len, "/%c/%s/", hash, usercopy);
-            free(usercopy);
+            len += snprintf(fname+len, size-len, "/%c/%s/", hash, user);
 
             if (!script) { /* default script */
                 char *bc_fname;
